.include "2313def.inc"

;*****************************************************************************************
;	Variablen
;*****************************************************************************************
.def	Current			= r12	; Current value from converter
.def	LcdHunderter	= r13	; Registers for Hex--> Dez-conversion
.def	LcdZehner		= r14
.def	LcdEinser		= r15
.def	DelayTemp		= r16	; Temporary Variable for Delay Routine
.def	Temp			= r17	; Temporary Variable
.def	Temp2			= r18	; Temporary Variable
.def	Menu			= r19	; Menu Configuration
;			Bits 10	:
;				00 : Stufe select
;				01 : Drehzahl select
;				10 : I_Up select
;				11 : I_Down select
;			Bit 7 :
;				0 : Normal Mode
;				1 : Menu active
;			Bit 6: Menu Mode
;				0 : Show/Select Item
;				1 : Change Value
.def	Temp3			= r20	; Temporary Variable

;*****************************************************************************************
;	Werte
;*****************************************************************************************
.equ	LCD_PORT     	= PORTB	
.equ	LCD_DDR	     	= DDRB
.equ	LCD_PIN	     	= PINB
.equ	LCD_CRTL     	= PORTD
.equ	LCD_CRTL_DDR 	= DDRD
.equ	LCD_CRTL_PIN 	= PIND
.equ	LCD_CRTL_E	 	= 2
.equ	LCD_CRTL_RS	 	= 3
.equ	Trans8Bit		= 0b00000011
.equ	Trans4Bit		= 0b00000010
.equ	ClearDisp		= 0b00000001
.equ	Start1Line		= 0b00000010
.equ	Start2Line		= 0b11000000
			;	Set the Display
				;              +------ Data:  0 = 4-bit; 1 = 8-bit
				;              |+----- Lines: 0 = 1; 1 = 2
				;              ||+---- Font:  0 = 5x8; 1 = 5x11
.equ	SetDisp			= 0b00101100
		; ***  'CURSOR/DISPLAY SHIFT' command
				;               +----- S/C:  0 = cursor; 1 = display
				;		        |+---- R/L:  0 = left; 1 = right
.equ	ShCursRight		= 0b00010100
		; ***  'DISPLAY ON/OFF' command
				;                +---- Display: 0 = off; 1 = on
				;                |+--- Cursor: 0 = off; 1 = on
				;                ||+-- Blink: 0 = off; 1 = on
.equ	DispOn			= 0b00001100
		; *** 'ENTRY MODE' command
			;					  +--- Direction: 0 = left; 1 = right
			;					  |+-- Shift Dislay: 0 = off; 1 = on
.equ	ShiftCursRight	= 0b00000110
.equ	StufeMax		= 15
.equ	DrehzahlMax		= 60
.equ	I_UpMax			= 245
.equ	I_DownMax		= 254

;*****************************************************************************************
;	SRAM Variablen
;*****************************************************************************************
.equ	Stufe		= 0x60
.equ	Drehzahl	= 0x61
.equ	I_Up		= 0x62
.equ	I_Down		= 0x63
.equ	Drehzahl0	= 0x64
.equ	NumLcdN1	= 0x65
.equ	NumLcdN2	= 0x66
.equ	NumLcdN3	= 0x67
.equ	NumLcdN4	= 0x68
.equ	NumLcdN5	= 0x69
.equ	NumLcdN6	= 0x6a
.equ	NumLcdN7	= 0x6b
.equ	NumLcdN8	= 0x6c

;*****************************************************************************************
;	EEPROM Variablen
;*****************************************************************************************
.eseg

		.dw	0x0000			; Drehzahl  2
		.dw	0x0001			; Drehzahl  3
		.dw	0x0002			; Drehzahl  4
		.dw	0x0003			; Drehzahl  5
		.dw	0x0004			; Drehzahl  6
		.dw	0x0005			; Drehzahl  7
		.dw	0x0006			; Drehzahl  8
		.dw	0x0007			; Drehzahl  9
		.dw	0x0008			; Drehzahl 10
		.dw	0x0009			; Drehzahl 11
		.dw	0x000a			; Drehzahl 12
		.dw	0x000b			; Drehzahl 13
		.dw	0x000c			; Drehzahl 14
		.dw	0x000d			; Drehzahl 15
		.dw	0x000e			; Drehzahl 16
		.dw 0x000f			; I_Up
		.dw 0x0010			; I_Down

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************
		rjmp	START	; Reset Vektor
		reti			; Int0 Pin
		reti			; Int1 Pin
		reti			; Timer1 Capture Event
		reti			; Timer1 Compare Event
		reti			; Timer1 Overflow
		reti			; Timer0 Overflow
		rjmp	RX_CURRENT	; UART RX Complete
		reti			; UART Data Register Empty
		reti			; UART TX Complete
		reti			; Analog Comparator

;*****************************************************************************************
;	Daten fr das Display
;*****************************************************************************************
Clean:		.db "        "
MenuText:	.db "Stufe   "
			.db "Drehzahl"
			.db " I Up   "
			.db " I Down "
Welcome:	.db "SF Umric"
			.db "hter V1 "
SetTxt:		.db "  Set.. "
PleaseTxt:	.db " Please "
SelectTxt:	.db "select  "

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************
START:
		ldi		Temp, low(RAMEND)		; Initialize Stackpointer
    	out   	SPL, Temp
		cli								; Disable all Interrupts
    	;ldi   	Temp, high(RAMEND)
    	;out   	SPH, temp
		ldi		Temp, 0b00001111		; enable the watchdog timer with 1,9s cycle
		wdr
		out		WDTCR, Temp
		clr		Temp
		out		GIMSK, Temp				; Disable Interrupts Int0 and Int1
		out		TIMSK, Temp				; Disable all Timer Interrupts
		ldi		Temp, 0b00001100		; Set LCD Control Pins as output
		out		LCD_CRTL_DDR, Temp		; set up the transfer mode for UART
		ldi		Temp, 100
		out		UBRR, Temp				; set the Baud Rate 9600 at 4,433MHz Clock
		ldi		Temp, 0b10011000
		out		UCR, Temp				; enable serial input and output and RXC int
		rcall	DELAY4ms				; Time for the Display to start up
		rcall	DISP_TRANS				; Set the Transfer mode
		ldi		Temp, ClearDisp			; Clear the Display
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		Temp, SetDisp			; Set Up the Display Lines and Fonts
		rcall	LCD_SEND_CMD
		ldi		Temp, ShCursRight		; Set Display/Cursor Shift and Direction
		rcall	LCD_SEND_CMD
		ldi		Temp, DispOn			; Set Display/Cursor ON/OFF
		rcall	LCD_SEND_CMD
		ldi		Temp, ShiftCursRight	; Set the Cursor/Display Right/left Shift during Entry
		rcall	LCD_SEND_CMD
		ldi		Temp, Start1Line		; Send Cursor to Beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZL, low(Welcome*2)		; Set Pointer to Menu Text
		ldi		ZH, high(Welcome*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print the Text
		ldi		Temp, Start2Line		; Send Cursor to beginning of second Line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print the next Line
		wdr
		rcall	DELAY1s					; Wait 1s
		wdr
		clr		Temp					;transmit zero drehzahl
		ldi		Temp2, 0x02
		rcall	TX_VALUE
		ldi		Temp, 0x0f
		out		EEAR, Temp
		rcall	READ_EEPROM				; restore I_Up value
		in		Temp, EEDR
		sts		I_Up, Temp
		ldi		Temp2, 0x03				; transmit I_up to SAA-controller
		rcall	TX_VALUE
		ldi		Temp, 0x10
		out		EEAR, Temp
		rcall	READ_EEPROM				; restore I_Down value
		in		Temp, EEDR
		sts		I_Down, Temp
		ldi		Temp2, 0x04				; transmit I_down to SAA-controller
		rcall	TX_VALUE
		ldi		Temp, Start1Line		; Send Cursor to Beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms				; *** Always use this Delay between a command and the Text entry!!
		ldi		ZL, low(PleaseTxt*2)	; Set Pointer to Menu Text
		ldi		ZH, high(PleaseTxt*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print text
		ldi		ZL, low(SelectTxt*2)	; Set Pointer to Menu Text
		ldi		ZH, high(SelectTxt*2)
		ldi		Temp, Start2Line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print second Line
		sei								; enable global interrupts

forever:
		wdr
		rcall	DELAY4ms
		sbis	LCD_PIN, 2				; ask for the Menu Button
		rcall	MENU_ROUTINE
		rjmp	forever					; end program here


;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************
MENU_ROUTINE:						; *** Main Menu Routine to chose menu entries
		cli								; disable	interrupts
		ori		Menu, 0b10000000		; set the menu active bit
		andi	Menu, 0b11111100		; activate the Stufe entry
		ldi		Temp, Start1Line		; send Cursor to beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZL, low(MenuText*2)		; Set Pointer to Menu Text
		ldi		ZH, high(MenuText*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print Menu text
		rcall	BUT2REL					; Wait until Menu button is released
 MENU_ROUTINE_INNER:				; Loop until Menu is deactivated
		rcall	DELAY4ms
		sbis	LCD_PIN, 0				; Up Button pressed?
		rcall	MENU_UP					; Yes
		sbis	LCD_PIN, 1				; Select Button pressed?
		rcall	MENU_SEL				; Yes
		sbis	LCD_PIN, 3				; Down Button pressed?
		rcall	MENU_DOWN				; Yes
		wdr
		sbic	LCD_PIN, 2				; Menu Button NOT pressed?
		rjmp	MENU_ROUTINE_INNER		; not pressed
									; Exit main menu routine
		rcall	BUT2REL					; Wait for release of menu button
		wdr
		andi	Menu, 0b01111111		; clear menu active bit
		ldi		Temp, Start1Line		; set cursor to beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZH, 0					; set pointer to Drehzahl
		ldi		ZL, Drehzahl
		ld		Temp, Z					; put Drehzahl value into Temp
		rcall	MENU_DREHZAHL_OUT		; print the actual Drehzahl value
		ldi		Temp, Start2Line		; set cursor to beginning of second line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZL, low(Clean*2)		; Set Pointer to Menu Text
		ldi		ZH, high(Clean*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; print line
		sei								; enable interrupts
		ret								; exit main menu

MENU_DOWN:							; *** Routine to go to the previous menu entry
;		push	Temp
		ldi		Temp, 0b00000011		; only use the last two bits for compare
		and		Temp, Menu
		cpi		Temp, 0b00000000		; menu entry already zero?
		breq	MENU_DOWN_OUT			; yes, don't decrease and exit here
		dec		Menu					; no, decrease
		sbiw	ZL, 16					; set pointer to beginning of previous menu text
		ldi		Temp, Start1Line		; set cursor to beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; print text
		ldi		Temp, Start2Line		; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		push	ZH
		push	ZL
		ldi		ZH, high(Clean*2)		; clear line 2
		ldi		ZL, low(Clean*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM
		pop		ZL
		pop		ZH
 MENU_DOWN_OUT:							; exit after down button is released
		rcall	BUT3REL
;		pop		Temp
		ret

MENU_UP:							; *** Routine to go to the previous menu entry
;		push	Temp
		ldi		Temp, 0b00000011		; only use the last two bits for compare
		and		Temp, Menu
		cpi		Temp, 0b00000011		; menu entry already 3?
		breq	MENU_UP_OUT				; yes, don't decrease and exit here
		inc		Menu					; no, increase
		ldi		Temp, Start1Line		; set cursor to beginning of first line
		rcall	LCD_SEND_CMD			; pointer is already at the beginning of the next entry
		rcall	DELAY4ms
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; print text
		ldi		Temp, Start2Line		; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		push	ZH
		push	ZL
		ldi		ZH, high(Clean*2)		; clear line 2
		ldi		ZL, low(Clean*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM
		pop		ZL
		pop		ZH
 MENU_UP_OUT:							; exit after up button is released
		rcall	BUT0REL
;		pop		Temp
		ret

MENU_SEL:							; *** Routine to change a value of a menu entry
		rcall	BUT1REL
		wdr
;		push	Temp
;		push	Temp2
		push	ZH						; save the Z pointer because is is used by the other menu routines
		push	ZL
		mov		Temp, Menu				; backup of menu variable
		andi	Temp, 0b00000011		; only use the last two bits
		mov		Temp2, Temp				; next backup
		ldi		XH, 0
		mov		XL, Temp				; X Pointer to SRAM Register last two bits
		adiw	XL, 0x30				; increase X by 60 to point into the correct area
		adiw	XL, 0x30
		ld		Temp, X					; Load SRAM Value into Temp
		cpi		Temp2, 0b00000000		; entry = Stufe?
		brne	SEL_NO_STUFE			; no, branch
		ldi		Temp3, StufeMax			; load upper limit
 MENU_SEL_STUFE_INNER:				;	  Routine to change the value of stufe
		rcall	MENU_STUFE_OUT			; print actual Stufe value
		rcall	DELAY4ms
		sbis	LCD_PIN, 0				; Up Button pressed?
		rcall	VALUE_UP				; Yes
		sbis	LCD_PIN, 3				; Down Button pressed?
		rcall	VALUE_DOWN				; Yes
		sbis	LCD_PIN, 2				; Menu Button NOT pressed?
		rjmp	MENU_SEL_STUFE_ESCAPE	; Exit Sel_Stufe without save
		wdr
		sbic	LCD_PIN, 1				; Select Button NOT pressed?
 		rjmp	MENU_SEL_STUFE_INNER	; not pressed, stay in loop
		sts		Stufe, Temp				; store actual value into SRAM
		ldi		Temp, Start2Line		; send cursor to beginning of line 2
		rcall	LCD_SEND_CMD
		ldi		ZL, low(SetTxt*2)		; print the text Set for one second
		ldi		ZH, high(SetTxt*2)
		rcall	LCD_PRINT_PM
		rcall	DELAY1s
		wdr
 MENU_SEL_STUFE_ESCAPE:				;	  Routine to Exit Sel_Stufe without save
		ld		Temp, X					; recall Stufe from SRAM
		rcall	LOAD_DREHZAHL			; load the Drehzahl value from the correct register
		ldi		Temp2, 2				; send the corresponding drehzahl value to SAA-Controller
		lds		Temp, Drehzahl
		rcall	TX_VALUE
		ld		Temp, X
		ldi		Temp2,8
		rcall	MENU_STUFE_OUT			; print actual Stufe value again
 		rjmp	SEL_NO_I_DOWN			; exit
 SEL_NO_STUFE:
		cpi		Temp2, 0b00000001		; entry = Drehzahl?
		brne	SEL_NO_DREHZAHL			; no, branch
		ldi		Temp3, DrehzahlMax		; load upper limit
 MENU_SEL_DREHZAHL_INNER:			;	  Routine to change the value of Drehzahl
 		push	Temp
 		ldi		Temp, Start2Line		; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		pop		Temp
		rcall	DELAY4ms
		rcall	MENU_DREHZAHL_OUT		; print actual Drehzahl value
		rcall	DELAY4ms
		sbis	LCD_PIN, 0				; Up Button pressed?
		rcall	VALUE_UP				; Yes
		sbis	LCD_PIN, 3				; Down Button pressed?
		rcall	VALUE_DOWN				; Yes
		sbis	LCD_PIN, 2				; Menu Button NOT pressed?
		rjmp	MENU_SEL_DREHZAHL_ESCAPE; Exit Sel_Drehzahl without save
		wdr
		sbic	LCD_PIN, 1				; Select Button NOT pressed?
 		rjmp	MENU_SEL_DREHZAHL_INNER	; not pressed, stay in loop
		sts		Drehzahl, Temp			; store actual value into SRAM
		rcall	SAVE_DREHZAHL			; save Drehzahl value into the correct register
		ldi		Temp, Start2Line		; send cursor to beginning of line 2
		rcall	LCD_SEND_CMD
		ldi		ZL, low(SetTxt*2)		; print the text Set for one second
		ldi		ZH, high(SetTxt*2)
		rcall	LCD_PRINT_PM
		ldi		Temp2, 8
		rcall	DELAY1s
		wdr
 MENU_SEL_DREHZAHL_ESCAPE:			;	  Routine to Exit Sel_Drehzahl without save
		ld		Temp, X					; recall Drehzahl from SRAM
		push	Temp2
		ldi		Temp2, 0x02				; Drehzahl marker
		rcall	TX_VALUE				; send the values of Temp and Temp2
		pop		Temp2
 		push	Temp
 		ldi		Temp, Start2Line		; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		pop		Temp
		rcall	MENU_DREHZAHL_OUT		; print actual Drehzahl value again
 		rjmp	SEL_NO_I_DOWN			; exit
 SEL_NO_DREHZAHL:
		cpi		Temp2, 0b00000010		; entry = I_Up?
		brne	SEL_NO_I_UP				; no, branch
		ldi		Temp3, I_UpMax			; load upper limit
 MENU_SEL_I_UP_INNER:				;	  Routine to change the value of stufe
		rcall	MENU_I_OUT				; print actual Current value
		rcall	DELAY4ms
		sbis	LCD_PIN, 0				; Up Button pressed?
		rcall	VALUE_UP				; Yes
		sbis	LCD_PIN, 3				; Down Button pressed?
		rcall	VALUE_DOWN				; Yes
		sbis	LCD_PIN, 2				; Menu Button NOT pressed?
		rjmp	MENU_SEL_I_UP_ESCAPE	; Exit Sel_I_Up without save
		wdr
		sbic	LCD_PIN, 1				; Select Button NOT pressed?
 		rjmp	MENU_SEL_I_UP_INNER		; not pressed, stay in loop
		sts		I_Up, Temp				; store actual value into SRAM
		out		EEDR, Temp
		ldi		Temp2, 0x0f				; Write Value to EEPROM
		out		EEAR, Temp2
		rcall	WRITE_EEPROM
		ldi		Temp, Start2Line		; send cursor to beginning of line 2
		rcall	LCD_SEND_CMD
		ldi		ZL, low(SetTxt*2)		; print the text Set for one second
		ldi		ZH, high(SetTxt*2)
		rcall	LCD_PRINT_PM
		rcall	DELAY1s
		wdr
 MENU_SEL_I_UP_ESCAPE:				;	  Routine to Exit Sel_I_Up without save
		ld		Temp, X					; recall I_up from SRAM
		ldi		Temp2, 0x03
		rcall	TX_VALUE				; transmit current_up value
		ldi		Temp2, 8
		rcall	MENU_I_OUT				; print actual I_Up value again
 		rjmp	SEL_NO_I_DOWN			; exit
 SEL_NO_I_UP:
		cpi		Temp2, 0b00000011		; entry = I_Down?
		brne	SEL_NO_I_DOWN			; no, branch
		ldi		Temp3, I_DownMax		; load upper limit
 MENU_SEL_I_DOWN_INNER:				;	  Routine to change the value of stufe
		rcall	MENU_I_OUT				; print actual Current value
		rcall	DELAY4ms
		sbis	LCD_PIN, 0				; Up Button pressed?
		rcall	VALUE_UP				; Yes
		sbis	LCD_PIN, 3				; Down Button pressed?
		rcall	VALUE_DOWN				; Yes
		sbis	LCD_PIN, 2				; Menu Button NOT pressed?
		rjmp	MENU_SEL_I_DOWN_ESCAPE	; Exit Sel_I_Down without save
		wdr
		sbic	LCD_PIN, 1				; Select Button NOT pressed?
 		rjmp	MENU_SEL_I_DOWN_INNER	; not pressed, stay in loop
		sts		I_Down, Temp			; store actual value into SRAM
		out		EEDR, Temp
		ldi		Temp2, 0x10				; Write Value to EEPROM
		out		EEAR, Temp2
		rcall	WRITE_EEPROM
		ldi		Temp, Start2Line		; send cursor to beginning of line 2
		rcall	LCD_SEND_CMD
		ldi		ZL, low(SetTxt*2)		; print the text Set for one second
		ldi		ZH, high(SetTxt*2)
		rcall	LCD_PRINT_PM
		rcall	DELAY1s
		wdr
 MENU_SEL_I_DOWN_ESCAPE:			;	  Routine to Exit Sel_I_Up without save
		ld		Temp, X					; recall Current from SRAM
		ldi		Temp2, 0x04				; transmit current_down value
		rcall	TX_VALUE
		ldi		Temp2, 8
		rcall	MENU_I_OUT				; print actual I_Up value again
 		rjmp	SEL_NO_I_DOWN			; exit
 SEL_NO_I_DOWN:
 		rcall	BUT1REL				; wait for release of set button
		rcall	BUT2REL				; wait for release of menu button
		wdr
		pop		ZL					; exit the change value routine
		pop		ZH
;		pop		Temp2
;		pop		Temp
		ret

MENU_STUFE_OUT:					; *** Print the Number stored in Temp as Stufe
		push	Temp				; Temp is used by the Change value subroutines
		push	Temp2				; Temp2 is used by MENU_SEL routine
		rcall	HEX_TO_DEZ			; convert into ascii printables
		ldi		Temp, 0x53			; load "S", some spaces and the ascii number into NumLcd...
		sts		NumLcdN1, Temp
		ldi		Temp, 0x20
		sts		NumLcdN2, Temp
		sts		NumLcdN3, Temp
		sts		NumLcdN4, Temp
		sts		NumLcdN5, Temp
		sts		NumLcdN6, Temp
		mov		Temp, LcdZehner
		sts		NumLcdN7, Temp
		mov		Temp, LcdEinser
		sts		NumLcdN8, Temp
		ldi		Temp, Start2Line	; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZH, 0x00			; set Z pointer to NumLcd
		ldi		ZL, NumLcdN1
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM		; print it
		pop		Temp2
		pop		Temp
		ret

MENU_DREHZAHL_OUT:				; *** Print the Number stored in Temp as Stufe
		push	Temp				; Temp is used by the Change value subroutines
		push	Temp2				; Temp2 is used by MENU_SEL routine
		rcall	HEX_TO_DEZ			; convert into ascii printables
		ldi		Temp, 0x20			; load the ascii number and some zeros into NumLcd...
		sts		NumLcdN1, Temp
		sts		NumLcdN2, Temp
		sts		NumLcdN8, Temp
		mov		Temp, LcdZehner
		sts		NumLcdN3, Temp
		mov		Temp, LcdEinser
		sts		NumLcdN4, Temp
		ldi		Temp, 0x30
		sts		NumLcdN5, Temp
		sts		NumLcdN6, Temp
		sts		NumLcdN7, Temp
		ldi		ZH, 0x00			; set Z pointer to NumLcd
		ldi		ZL, NumLcdN1
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM		; print it
		pop		Temp2
		pop		Temp
		ret

LOAD_DREHZAHL:					; *** load Drehzahl value from a register selected by Stufe to
									; Register Drehzahl
		push	Temp
		push	Temp2
		lds		Temp, Stufe			; load actual Stufe value
		cpi		Temp, 0x00			; is 0 selected?
		brne	DREHZAHL_FROM_EEPROM; no, load from EEPROM
		lds		Temp2, Drehzahl0	; load Drehzahl0 from SRAM
		rjmp	LOAD_DREHZAHL_OUT	; exit
 DREHZAHL_FROM_EEPROM:
 		subi	Temp, 1				; Stufe 1 is stored in eeprom 0
		out		EEAR, Temp			; Load corresponding Drehzahl value
		rcall	READ_EEPROM
		in		Temp2, EEDR			; put value to Temp2
 LOAD_DREHZAHL_OUT:
		sts		Drehzahl, Temp2		; and store as actual value into SRAM
 		pop		Temp2
		pop		Temp
		ret

SAVE_DREHZAHL:					; *** Save Drehzahl value from register Drehzahl to a register
									; selected by the value of Stufe
		push Temp
		push Temp2
		lds		Temp, Stufe			; load actual Stufe value
		lds		Temp2, Drehzahl		; load actual Drehzahl value
		cpi		Temp, 0x00			; is Stufe 0 selected?
		brne	DREHZAHL_TO_EEPROM	; no, save to EEPROM
		sts		Drehzahl0, Temp2	; store value to SRAM as Drehzahl0
		rjmp	LOAD_DREHZAHL_OUT		; exit
 DREHZAHL_TO_EEPROM:
 		subi	Temp, 1			; Stufe 1 is stored in eeprom 0
		out		EEAR, Temp		; use corresponding Stufe adress
		out		EEDR, Temp2		; Store Drehzahl there
		rcall	WRITE_EEPROM
 SAVE_DREHZAHL_OUT:
		pop Temp2
		pop Temp
		ret

MENU_I_OUT:						; *** Print the Number stored in Temp as Current
		push	Temp				; Temp is used by the Change value subroutines
		push	Temp2				; Temp2 is used by MENU_SEL routine
		rcall	HEX_TO_DEZ			; convert into ascii printables
		ldi		Temp, 0x49			; load "I", and the ascii number
		sts		NumLcdN1, Temp
		ldi		Temp, 0x20
		sts		NumLcdN2, Temp
		sts		NumLcdN3, Temp
		sts		NumLcdN4, Temp
		sts		NumLcdN5, Temp
		mov		Temp, LcdHunderter
		sts		NumLcdN6, Temp
		mov		Temp, LcdZehner
		sts		NumLcdN7, Temp
		mov		Temp, LcdEinser
		sts		NumLcdN8, Temp
		ldi		Temp, Start2Line	; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZH, 0x00			; set Z pointer to NumLcd
		ldi		ZL, NumLcdN1
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM		; print it
		pop		Temp2
		pop		Temp
		ret

MENU_I_DOWN_OUT:
		push	Temp2				; Temp2 is used by MENU_SEL routine
		mov		temp, temp2			; store menu variable last two bits into temp
		rcall	LCD_CONV_NUM		; convert Temp into display number and store into SRAM65
		rcall	LCD_PRINT_65		; print SRAM65
		pop		Temp2
		ret

HEX_TO_DEZ:						; *** Converts a hex number into a decimal one
									; put the hex number into Temp
									; after call the decimal number is in r13,r14,r15 already printable
									; as ascii to LCD. and Z points to r13.
		push	Temp				; Temp is used by the Change value subroutines
		push	Temp2				; Temp2 is used by MENU_SEL routine
		ldi		ZH, 0
		ldi		ZL, 15				; Place  Z pointer to store Einser Value
		rcall	HEX_DEZ_CONV		; convert last digit
		mov		Temp, Temp2			; use Zehner value
		rcall	HEX_DEZ_CONV		; convert the rest
		ldi		Temp, 0x30
		add		LcdEinser, Temp		; convert last digit into ascii
		add		LcdZehner, Temp		; convert Zehner into ascii
		ldi		Temp, 0x20
		add		LcdHunderter, Temp	; add the space sign to Zehner and Hunderter Digit
		cp		LcdHunderter, Temp	; compare if Hunderter = "space"
		breq	KILL_ZEROS			; yes, kill zeros of zehner too
		ldi		Temp, 0x10			; no, convert Hunderter into ascii
		add		LcdHunderter, Temp
		rjmp	KILL_GO_OUT			; exit
 KILL_ZEROS:						; kill zeros of zehner
		ldi		Temp, 0x30
		cp		LcdZehner, Temp		; is Zehner = ascii zero?
		brne	KILL_GO_OUT			; no, don't change and exit
		ldi		Temp, 0x10			; yes, convert to space
		sub		LcdZehner, Temp
 KILL_GO_OUT:
		pop		Temp2
		pop		Temp
		ret

HEX_DEZ_CONV:					; *** Convert a hex number into a decimal one:
									; place the hex number into Temp
									; place the pointer to Einser value
									; after one convert Z is pointing to Zener value
									; and Einser value is converted.
									; if the temp value is larger then 99 conversion is required
									; to call two times. Sequence of the registers must be Hunderter, Zener, Einser
									;													(least)				(highest)
		clr		Temp2				; next decade = 0
 HEX_DEZ_INNER:
 		cpi		Temp, 10			; is Temp >= than 10?
		brcs	HEX_DEZ_OUT			; no, exit
		subi	Temp, 10			; yes, subtract 10
		inc		Temp2				; increase Temp2 (10er value)
		rjmp	HEX_DEZ_INNER		; and try again
 HEX_DEZ_OUT:
 		st		Z, Temp				; Store value
		sbiw	ZL, 1				; place pointer to the next decade
		st		Z, Temp2			; Store Next Decade
		ret

VALUE_UP:						; *** inrease the value of Temp if smaller than the value of Temp3 and return
		cpse	Temp, Temp3			; test if equal
		inc		Temp				; not equal
		rcall	BUT0REL				; wait for release of Up Button
		ret

VALUE_DOWN:						; *** decrease the value of Temp if larger than zero
		push	Temp2				; Temp2 is used by MENU_SEL Routine
		clr		Temp2
		cpse	Temp, Temp2			; test if zero
		dec		Temp				; not zero
		rcall	BUT3REL				; wait for release of Up Button
		pop		Temp2
		ret

BUT0REL:						; *** Wait until Button 0 is released
		sbis	LCD_PIN, 0
		rjmp	BUT0REL
		ret

BUT1REL:						; *** Wait until Button 1 is released
		sbis	LCD_PIN, 1
		rjmp	BUT1REL
		ret

BUT2REL:						; *** Wait until Button 2 is released
		sbis	LCD_PIN, 2
		rjmp	BUT2REL
		ret

BUT3REL:						; *** Wait until Button 3 is released
		sbis	LCD_PIN, 3
		rjmp	BUT3REL
		ret

RX_CURRENT:
		cli								; disable global interrupts
		push	Temp
		push	Temp2
		push	ZL
		push	ZH
		in		Temp, UDR				; Read data
		sbic	USR, FE					; test if frame error
		rjmp	RX_OUT					; go out if Frame error occured
		sbic	USR, OR
		rjmp	RX_OUT
		cpi		Temp, 255
		breq	TX_ALL
		cpi		Temp, 254				; Test if current start character sent
		brne	RX_OUT					; no, go out
 RX_WAIT_2:
 		sbis	USR, RXC				; wait until the next character received
		rjmp	RX_WAIT_2
		in		Temp, UDR				; read value
		ldi		Temp2, 0x30				; for conversion into LCD Numeric
		add 	Temp, Temp2				; conversion
		sts		NumLcdN5, Temp
 RX_WAIT_3:
 		sbis	USR, RXC				; wait until the next character received
		rjmp	RX_WAIT_3
		in		Temp, UDR				; read value
		ldi		Temp2, 0x30				; for conversion into LCD Numeric
		add 	Temp, Temp2				; conversion
		sts		NumLcdN6, Temp
 RX_WAIT_4:
 		sbis	USR, RXC				; wait until the next character received
		rjmp	RX_WAIT_4
		in		Temp, UDR				; read value
		ldi		Temp2, 0x30				; for conversion into LCD Numeric
		add 	Temp, Temp2				; conversion
		sts		NumLcdN7, Temp
 RX_WAIT_5:
 		sbis	USR, RXC				; wait until the next character received
		rjmp	RX_WAIT_5
		in		Temp, UDR				; read value
		ldi		Temp2, 0x30				; for conversion into LCD Numeric
		add 	Temp, Temp2				; conversion
		sts		NumLcdN8, Temp
		ldi		Temp2, 0x20				; space character
		sts		NumLcdN4, Temp2
		sts		NumLcdN3, Temp2
		sts		NumLcdN2, Temp2
		sts		NumLcdN1, Temp2
		ldi		Temp, Start2Line		; send cursor to beginning of second line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZH, 0x00				; set Z pointer to NumLcd
		ldi		ZL, NumLcdN1
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM			; print it
 RX_OUT:
		pop		ZH
		pop		ZL
		pop		Temp2
		pop		Temp
		sei								; enable global interrupts
		reti
 TX_ALL:
 		rcall	DELAY4ms
		push	ZH
		push	ZL
		ldi		ZH, 0					; set pointer to Drehzahl
		ldi		ZL, Drehzahl
		ld		Temp, Z	
 		ldi		Temp2, 2
		rcall	TX_VALUE
		lds		Temp, I_Up
		ldi		Temp2, 3				; transmit I_up to SAA-controller
		rcall	TX_VALUE
		lds		Temp, I_Down
		ldi		Temp2, 4				; transmit I_down to SAA-controller
		pop		ZL
		pop		ZH
		rcall	TX_VALUE
 		rjmp	RX_OUT

TX_VALUE:						; *** Transmit Value to converter via UART
									; transmits first the value 255, then the value
									; in Temp2 and finally Temp
		push	Temp
 TX_255_WAIT_BEGIN:
 		sbis	USR, UDRE			; run the loop until UDR empty Flag is set.
 		rjmp	TX_255_WAIT_BEGIN
		ldi		Temp, 255
		out		UDR, Temp			; send 255
 TX_TEMP2_WAIT_BEGIN:
 		sbis	USR, UDRE			; run the loop until UDR empty Flag is set.
 		rjmp	TX_TEMP2_WAIT_BEGIN
		out		UDR, Temp2			; send Temp2
		pop		Temp
 TX_TEMP_WAIT_BEGIN:
 		sbis	USR, UDRE			; run the loop until UDR empty Flag is set.
 		rjmp	TX_TEMP_WAIT_BEGIN
 		out	UDR, Temp				; Send Temp
		ret

READ_EEPROM:					; *** read out the eeprom. Put the address into EEAR(6..0) and call.
									; after execution Data is put into EEDR
									; !!! Routine disables the GIE-bit!!!
		sbic	EECR, EEWE
		rjmp	READ_EEPROM			; wait until EEWE=0
		push	Temp
		in		Temp, SREG			; Save GIE Bit
		cli							; clear global interrupt enable
		sbi		EECR, EERE			; Set Read Bit
		out		SREG, Temp			; Restore GIE Bit
		pop		Temp
		ret

WRITE_EEPROM:					; *** write to eeprom. Put the address into EEAR(6..0) and call.
									; and data into EEDR.
									; !!! Routine disables the GIE-bit!!!
		sbic	EECR, EEWE
		rjmp	WRITE_EEPROM		; wait until EEWE=0
		push	Temp
		push	Temp2
		in		Temp, SREG			; Save GIE Bit
		clr		Temp2
		out		SREG, Temp2			; clear global interrupt enable
		sbi		EECR, EEMWE			; Set Write Master Bit
		nop
		sbi		EECR, EEWE			; Set Write Bit
		out		SREG, Temp			; Restore GIE Bit
		pop		Temp2
		pop		Temp
		ret

LCD_CONV_NUM:					; *** convert the value of temp into a display number and write into SRAM65
		ldi		ZH, 0x00			; set Z pointer to SRAM 65 address
		ldi		ZL, 0x65
		ldi		Temp2, 0x30			; add hex 30 to Temp
		add		Temp, Temp2
		st		Z, Temp				; store to SRAM
		ret

LCD_PRINT_65:
		ldi		Temp, Start2Line	; send cursor to beginning of line 2
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZH, 0x00			; set pointer to SRAM 65
		ldi		ZL, 0x65
		ldi		Temp2, 1
		rcall	LCD_PRINT_MEM		; print one sign
		ret

LCD_PRINT_PM:						; *** LCD_PrintPM: Prints from program memory
									; Put the starting memory location in Z (r31:r30)
									; Put the number of characters to print in Temp2
									; After execution, Z is at the character AFTER
									; the last to be printed and Temp2 is zero.
									; This function will not wrap if you the string
									; is bigger than the LCD.
		push	r0
		push	Temp
 LCD_PM_READ: 
		lpm
    	mov		Temp, r0
		rcall	LCD_SEND_CHAR
		adiw	r30, 1
		dec		Temp2
		brne	LCD_PM_READ
		pop		Temp
		pop		r0
		ret

LCD_PRINT_MEM:						; *** LCD_PrintMem: Prints from memory.
									; Put the starting memory location in Z (r31:r30)
									; Put the number of characters to print in Temp2
									; After execution, Z is at the character AFTER
									; the last to be printed and Temp2 is zero.
									; This function will not wrap if you the string
									; is bigger than the LCD.
		push	Temp
 LCD_MEM_READ: 
   		ld		Temp, Z+
		rcall	LCD_SEND_CHAR
		dec		Temp2
		brne	LCD_MEM_READ
		pop		Temp
		ret

LCD_SEND_CHAR:						; *** Send the Contents of Temp to the LCD using 4bit Mode
		push 	Temp
		push 	Temp2
		rcall	DISP_OUT
		mov 	Temp2, Temp			; Make a backup copy
		andi 	Temp2, 0b11110000	; Only use the upper nibble
		swap	Temp2				; move it to lower outputs
		out		LCD_PORT, Temp2		; Send it
		rcall	LCD_RS_HIGH
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		andi	Temp, 0b00001111	;use lower nibble of Temp
		out		LCD_PORT, Temp		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		rcall	LCD_RS_LOW			; Reset strobe signal
		rcall	DISP_IN
		pop 	Temp2
		pop 	Temp
		ret

DISP_TRANS:							; ************** Set the Transfer mode
		push	Temp
		rcall	DISP_OUT
		ldi		Temp, Trans8Bit		; set the Display in 8bit mode first
		out		LCD_PORT, Temp
		rcall	LCD_RS_LOW
		rcall	LCD_PULSEE			; Send
;		rcall	DELAY775			; Time for the Display
		cbi		LCD_PORT,0			; Now it's safe to go into 4-bit mode.
		rcall	LCD_PULSEE
		rcall 	DELAY775
		rcall	DISP_IN
		pop		Temp
		ret

LCD_SEND_CMD:						; ************** LCD_SendCmd: Routine to write a command
									; to the instruction register.
									; The value to be written should be stored in Temp.  
									; The value is sent 4 bits at a time.
		push 	Temp
;		push 	Temp2
		mov 	Temp2, Temp			; Make a backup copy
		rcall	DISP_OUT
		andi 	Temp2, 0b11110000	; Only use the upper nibble
		swap	Temp2				; move it to the lower ports
		out		LCD_PORT, Temp2		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
;		rcall	DELAY775
		andi 	Temp, 0b00001111	; Only use the lower nibble
		out		LCD_PORT, Temp		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		rcall	DISP_IN
;		pop 	Temp2
		pop 	Temp
		ret

LCD_RS_HIGH:	
		sbi		LCD_CRTL, LCD_CRTL_RS
		ret

LCD_RS_LOW:
		cbi		LCD_CRTL, LCD_CRTL_RS
		ret		

LCD_PULSEE:	
		sbi		LCD_CRTL, LCD_CRTL_E
		nop
		nop
		cbi		LCD_CRTL, LCD_CRTL_E
		ret

DISP_OUT:							; *** Set the LCD Data Pins as Output
		push	Temp
		ldi		Temp, 0b00001111
		out		LCD_DDR, Temp
		pop		Temp
		ret

DISP_IN:							; *** Set the LCD Data Pins as Input
		push	Temp
		ldi		Temp, 0b00000000
		out		LCD_DDR, Temp
		pop		Temp
		ret

DELAY1s:	;ret					; *** provide 3.971.125 MZ delay
		push	DelayTemp
		ldi		DelayTemp, 255
 DELAYCOUNT1s:
		rcall	DELAY4ms
		dec		DelayTemp
		brne	DELAYCOUNT1s
		pop		DelayTemp
		ret

;DELAY200ms:							; *** provide approx. 200ms Delay
;		push	DelayTemp
;		ldi		DelayTemp, 50
; DELAYCOUNT200ms:
; 		rcall	DELAY4ms
;		dec		DelayTemp
;		brne	DELAYCOUNT200ms
;		pop		DelayTemp
;		ret

DELAY4ms:
		;ret					; *** provide 15570 MZ delay
		push	DelayTemp
		ldi		DelayTemp, 20
 DELAYCOUNT4ms:
 		rcall	DELAY775
 		dec		DelayTemp
		brne	DELAYCOUNT4ms
		pop		DelayTemp
		ret

DELAY775:
		;ret					; *** provide 775 MZ delay including the rcall command
		push	DelayTemp				; 2MZ
		ldi		DelayTemp, 255			; 1MZ
 DELAYCOUNT:
 		dec		DelayTemp				; 1MZ
		brne	DELAYCOUNT				; 1/2MZ
		pop		DelayTemp				; 2MZ
		ret								; 3MZ, rcall 3MZ